/****************************************************************
 * pack.c
 *---------------------------------------------------------------
 * Copyright (C) 2014-2015 Osman Celimli
 * For conditions of distribution and use, see copyright notice in hscc.c
 ****************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "bytes.h"
#include "hu.h"
#include "music.h"
#include "pack.h"
/****************************************************************
 * Text Tables
 ****************************************************************/
char REM_PAKMode[]=		"PAK Mode...\n";
char REM_PAKDone[]=		"The current PAK binary is : ";

char ERR_NoPAKMem[]=	"ERROR- Out of memory\n";
char ERR_PAKEarlyEnd[]=	"ERROR- Unexpected end of include file\n";
char ERR_PAKBadWrite[]=	"ERROR- Couldn't write to output file\n";

/****************************************************************
 * int pak_buildBlock(FILE *pDest, FILE **ppSrcs, unsigned int uiFiles)
 ****************************************************************/
int pak_buildBlock(FILE *pDest, FILE **ppSrcs, unsigned int uiFiles)
{
	FILE *pCurFile = NULL;
	char *pacPack = NULL;
	unsigned int uiMusBase,uiPakLen;

	unsigned int *pauiMusLens;
	HuAddress ResAddr;
	unsigned int uiX,uiY,uiZ,uiS;
	int iStatus = -1;

	/**
	 * Calculate the required space for the track pack, including
	 * the header and all individual tracks. We'll store the size
	 * of each track in pauiMusLens[].
	 */
	if(NULL == (pauiMusLens = malloc(uiFiles * sizeof(unsigned int))))
	{
		fprintf(stderr, "%s", ERR_NoPAKMem);
		return -1;
	}

	uiMusBase = PACK_HEADER_OFFSET_ADDRESS + (uiFiles * PACK_TRACK_DESC_LEN);
	uiPakLen = uiMusBase;
	for(uiX = 0; uiX < uiFiles; uiX++)
	{
		pCurFile = ppSrcs[uiX];

		fseek(pCurFile, 0, SEEK_END);
		uiY = ftell(pCurFile);
		pauiMusLens[uiX] = uiY;
		uiPakLen += uiY;

		rewind(pCurFile);
	}

	if(NULL == (pacPack = malloc(uiPakLen * sizeof(char))))
	{
		free(pauiMusLens);
		fprintf(stderr, "%s", ERR_NoPAKMem);
		return -1;
	}

	/**
	 * Read each included music track into the package, stamping
	 * its offset in the directory aswell.
	 * If all goes well, we can then write out the pack.
	 */
	strcpy(pacPack + PACK_HEADER_OFFSET_MAGIC, PACK_MAGICWORD);
	pacPack[PACK_HEADER_OFFSET_COUNT] = uiFiles;

	uiY = PACK_HEADER_OFFSET_ADDRESS;
	uiZ = uiMusBase;
	for(uiX = 0; uiX < uiFiles; uiX++)
	{
		ResAddr = hu_calcAddr(uiZ);
		pacPack[uiY + PACK_TRACK_OFFSET_ADDRL] = ResAddr.ucLow;
		pacPack[uiY + PACK_TRACK_OFFSET_ADDRH] = ResAddr.ucHigh;
		pacPack[uiY + PACK_TRACK_OFFSET_ADDRB] = ResAddr.ucBank;
		pacPack[uiY + PACK_TRACK_OFFSET_ADDRM] = ResAddr.ucMap;

		uiS = fread(pacPack + uiZ, sizeof(char),
			pauiMusLens[uiX], ppSrcs[uiX]);
		if(uiS != pauiMusLens[uiX])
		{
			fprintf(stderr, "%s", ERR_PAKEarlyEnd);
			break;
		}

		uiY += PACK_TRACK_DESC_LEN;
		uiZ += pauiMusLens[uiX];
	}

	if(uiX == uiFiles)
	{
		fprintf(stdout, "%s%u bytes\n", REM_PAKDone, uiPakLen);
		uiY = fwrite(pacPack, sizeof(char), uiPakLen, pDest);
		if(uiY != uiPakLen)
		{
			fprintf(stderr, "%s", ERR_PAKBadWrite);
		}
		else
		{
			iStatus = 0;
		}
	}

	if(NULL != pauiMusLens) free(pauiMusLens);
	if(NULL != pacPack) free(pacPack);
	return iStatus;
}
