;****************************************************************
;		   Hu6270 VDC Helpful Defines			*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
; Ugamugh-ugh only got one background and 64 sprites, but very
; good at lifting data into VRAM.
; We're mapping this guy into the $100+ range to exclude it
; the from ZP addressing modes.
.define VDC_VRAM_SIZE		$8000

;****************************************************************
;		  VRAM Transfer Request Blocks			*
;****************************************************************
.define VREQ_TYPE		0
;		 76543210
; Tx Type Byte : ----iibb
; ii = Increment Mode
;	(00 = +1, 01 = +32, 10 = +64, 11 = +128)
; bb = Copy Mode / Bit Depth
;	00 = Planar 1bpp -> 4bpp BG Tile
;	01 = Planar 1bpp -> 2bpp BG Tile
;	10 = Direct Copy (word for word) 4bpp -> 4bpp
;	11 = Fast Direct Copy (jump to ST1/2s)
;
;	For 4bpp expansion, the various planes are expanded
; to the native PCE BG tile format with the unused planes zeroed
; out. Direct copy does what you expect, and bytes are copied in
; word for word. This should be used for non-pattern things like
; tilemap updates or copying over the SAT.
;----------------------------------------------------------------
.define VDC_TRANSFER_1BBG4	%00000000
.define VDC_TRANSFER_1BBG2	%00000001
.define VDC_TRANSFER_4BPP	%00000010
.define VDC_TRANSFER_DIRECT	%00000010
.define VDC_TRANSFER_FAST	%00000011
.define VDC_TRANSFER_MODEMASK	%00000011

.define VDC_TRANSFER_I1		%00000000
.define VDC_TRANSFER_I32	%00001000
.define VDC_TRANSFER_I64	%00010000
.define VDC_TRANSFER_I128	%00011000
.define VDC_TRANSFER_STEPMASK	%00011000

.define VDC_TRANSFER_FILL	%10000000

.define VREQ_SOURCE		1
.define VREQ_SOURCE_LO		VREQ_SOURCE + LO
.define VREQ_SOURCE_HI		VREQ_SOURCE + HI
.define VREQ_SOURCE_BANK	VREQ_SOURCE + BANK
.define VREQ_SOURCE_MCMAP	VREQ_SOURCE + MCMAP

.define VREQ_DEST		5
.define VREQ_DEST_LO		VREQ_DEST + LO
.define VREQ_DEST_HI		VREQ_DEST + HI

.define VREQ_SIZE		7
.define VREQ_SIZE_LO		VREQ_SIZE + LO
.define VREQ_SIZE_HI		VREQ_SIZE + HI

.define VREQ_LEN		VREQ_SIZE_HI + 1
.define VREQ_RINGLEN		128
.define VREQ_RINGMASK		$7F

;****************************************************************
;			 Register Map				*
;****************************************************************
; Exposed Registers
;-------------------
.define VDC_REG			$0100
; -R/
.define VDC_BSY			%01000000
.define VDC_VBL			%00100000
.define VDC_DV			%00010000
.define VDC_DS			%00001000
.define VDC_RR			%00000100
.define VDC_OR			%00000010
.define VDC_CR			%00000001
;	6: DMA currently active
;	5: VBL
;	4: VRAM to VRAM DMA Done
;	3: VRAM to SAT DMA Done
;	2: Scanline IRQ Occurred
;	1: 16-Sprite Overflow Occurred
;	0: Sprite 0 Collision Occurred
; -/W
; 	Select internal register for data r/w

.define VDC_DATALO		$0102
.define VDC_DATAHI		$0103
; -R/W
;	16-Bit data read and write port, final autoincrements
;	and writes are only performed when accessing VDC_DATAHI.


; Internal Registers
;--------------------
.define VDC_REG_MAWR		$0
; -/W
;	VRAM Write Pointer

.define VDC_REG_MARR		$1
; -/W
;	VRAM Read Pointer

.define VDC_REG_VxR		$2
; -R/W
;	VRAM Read/Write


.define VDC_REG_CR		$5
; -/W
.define	VDC_INCREMENT_1		%0000000000000000
.define	VDC_INCREMENT_32	%0000100000000000
.define	VDC_INCREMENT_64	%0001000000000000
.define	VDC_INCREMENT_128	%0001100000000000
;	12-11: Read/Write Autoincrement
;		(00= +1,01= +32,10= +64,11= +128) 
;	10-8: Set to 0
.define VDC_ENABLE_BG		%10000000
.define VDC_ENABLE_SPR		%01000000
.define VDC_IRQ_VBL		%00001000
.define VDC_IRQ_HBL		%00000100
.define VDC_IRQ_OVER		%00000010
.define VDC_IRQ_COL		%00000001
;	7: Background Enable
;	6: Sprite Enable
;	5-4: Set to 0
;	3: VBL IRQ Enable
;	2: HBL IRQ Enable
;	1: Sprite Overflow IRQ Enable
;	0: Sprite Collision IRQ Enable


.define VDC_REG_RCR		$6
; -/W
;	9-0: Raster Comparison Scanline, note that this is relative
;	to the start of the active display. Add 64 ($40) to your
;	target scanline before writing to this register. Values
;	below 64 or above (64 + frame height) will never cause an IRQ.

.define VDC_REG_BXR		$7
; -/W
;	9-0: Background X Scroll

.define VDC_REG_BYR		$8
; -/W
;	9-0: Background Y Scroll

.define VDC_REG_MWR		$9
; -/W
;	7: CG Mode
;		Used to select which two bitplanes will be used for
;		background tiles in slower VRAM modes (ignore).
.define VDC_MAP_32x32		%00000000
.define VDC_MAP_64x32		%00010000
.define VDC_MAP_128x32		%00100000
.define VDC_MAP_32x64		%01000000
.define VDC_MAP_64x64		%01010000
.define VDC_MAP_128x64		%01100000
;	6-4: Background Map Size
;		(000= 32x32,001= 64x32,010= 128x32,011= 128x32)
;		(100= 32x64,101= 64x64,110= 128x64,111= 128x64)
;	3-2: Sprite Dot Period
;	1-0: VRAM Dot Width
;		These should be left as zero unless we want to
;		use one of the strange 2bpp modes (not really).

.define VDC_REG_HSR		$A
; -/W
;	14-8: Horizontal Display Start
;	4-0: Horizontal Display Synchro

.define VDC_REG_HDR		$B
; -/W
;	14-8: Horizontal Display End
;	6-0: Horizontal Display Width

.define VDC_REG_VPR		$C
; -/W
;	15-8: Vertical Display Start
;	4-0: Vertical Synchro Width

.define VDC_REG_VDW		$D
; -/W
;	8-0: Vertical Display Width

.define VDC_REG_VCR		$E
; -/W
;	7-0: Vertical Display End


.define VDC_REG_DCR		$F
; -/W
;	4: SAT Auto-DMA Enable
;	3: Destination Data Direction (0=Increment,1=Decrement)
;	2: Source Data Direction (0=Increment,1=Decrement)
;	1: VRAM to VRAM Transfer Complete IRQ Enable
;	0: VRAM to SAT Transfer Complete IRQ Enable

.define VDC_REG_DMASRC		$10
; -/W
;	VRAM to VRAM DMA Source Address 

.define VDC_REG_DMADEST		$11
; -/W
;	VRAM to VRAM DMA Destination Address

.define VDC_REG_DMALEN		$12
; -/W
;	VRAM to VRAM DMA Transfer Length (words),
;	Transfer starts upon write to high byte	

.define VDC_REG_SATB		$13
; -/W
;	VRAM to SAT DMA Source Address
;	Transfer starts next VBL after write to high byte	

;****************************************************************
;			 Useful Macros				*
;****************************************************************
; VDC_Copy16 [REG] [SOURCE]
;	Copy a 16-Bit value to the destination VDC register.
;----------------------------------------------------------------
.macro VDC_Copy16
	ST0.b #\1
	LDA.w \2 + LO
	STA.w VDC_DATALO
	LDA.w \2 + HI
	STA.w VDC_DATAHI
.endm
