;****************************************************************
;		Run/Select Special Case Handling		*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
; These should be called every frame, right after reading
; the new controller state.

;****************************************************************
; HandlePause:							*
;	Check to see if the RUN button has been pressed and	*
; we're in a pausable state, if so- pause the game and wait for	*
; the button to be pressed again.				*
;****************************************************************
.section "HandlePause" force
HandlePause:
	LDA GLO_isPausable
	bne _checkFirstPress
	rts
_checkFirstPress
	LDA JOY_Pressed
	AND #GAMESTICK_RUN
	bne _wasPressed
	rts

_wasPressed
	LDA #$FF
	STA GLO_isPaused	; Set the game state to paused, and
	jsr HuSound_Pause	; tell the audio driver to shut up.
_waitNextPress
	jsr ReadJoy
	jsr HandleReset

	LDA JOY_Pressed		; Just wait for the second press of
	AND #GAMESTICK_RUN	; the RUN button, all our IRQs aside
	beq _waitNextPress	; from TIMER are still going, so we'll
				; effectively get a freeze frame.
	STZ GLO_isPaused
	jmp HuSound_UnPause
.ends

;****************************************************************
; HandleReset:							*
;	Check to see if the magic RUN + SEL combination is	*
; pressed, resetting the entire game if they are.		*
;****************************************************************
.section "HandleReset" force
HandleReset:
	LDA JOY_Current
	AND #(GAMESTICK_RUN | GAMESTICK_SELECT)
	CMP #(GAMESTICK_RUN | GAMESTICK_SELECT)
	beq _doReset
	rts

_doReset
	sei
	cld
	CLA
	TAX
	LDY SysRegion		; <- Don't forget to reload the region!
	jmp PCE_Reset		; MooZ - "Select + Run = Infinite Fun!"
.ends
