;****************************************************************
;		      Debug Text Printing			*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
; All of these write directly to VRAM, disabling IRQs beforehand.
;
; -DBG_Print:
;	Print a null terminated string
; -DBG_PrintBin:
;	Print a value in binary
; -DBG_PrintHex:
;	Print a value in hex
; -DBG_PrintTiles:
;	Print a null terminated tile string

;****************************************************************
; DBG_Print:							*
;	Print the null-terminated string whose address is in	*
; DBG_TextSrc to the horizontal strip of tiles at DBG_TextDest.	*
;****************************************************************
.section "DBG_Print" force
DBG_Print:
	PHY			; Make sure we don't conflict
	sei			; with VBL/HBL routines
				; communicating with the VDC.
	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	AND #%11100111		; Save the old VDC CR contents,
	STA VDC_DATAHI		; then force +1 increment mode.

	ST0 #VDC_REG_MAWR
	LDA DBG_TextDest + LO
	STA VDC_DATALO
	LDA DBG_TextDest + HI
	STA VDC_DATAHI

	ST0 #VDC_REG_VxR
	CLY
_writeText
	LDA (DBG_TextSrc),Y
	beq _done
	clc
	ADC #<(VCEL_SYSFONT - ' ')
	STA VDC_DATALO
	LDA #0
	ADC #>(VCEL_SYSFONT - ' ')
	STA VDC_DATAHI
	INY

	INC DBG_TextDest + LO	; Autoincrement the destination
	bne _noWrapSrc		; address for convenience.
	INC DBG_TextDest + HI
_noWrapSrc
	bra _writeText		; Copy letters one by one
_done

	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	STA VDC_DATAHI		; Restore old CR contents

	PLY
	cli			; IRQs back on...
	rts
.ends
;****************************************************************
; DBG_PrintBin:							*
;	Print A's value in binary as tiles at DBG_TextDest.	*
;****************************************************************
.section "DBG_PrintBin" force
DBG_PrintBin:
	PHY
	PHX
	TAX
	sei

	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	AND #%11100111
	STA VDC_DATAHI

	ST0 #VDC_REG_MAWR
	LDA DBG_TextDest + LO
	STA VDC_DATALO
	LDA DBG_TextDest + HI
	STA VDC_DATAHI

	ST0 #VDC_REG_VxR
	LDY #7
_writeBits
	TXA
	ROL
	TAX
	LDA #0
	ADC #<(VCEL_SYSFONT + ('0' - ' '))
	STA VDC_DATALO
	LDA #>(VCEL_SYSFONT + ('0' - ' '))
	STA VDC_DATAHI

	INC DBG_TextDest + LO
	bne _noWrapSrc
	INC DBG_TextDest + HI
_noWrapSrc
	DEY
	bpl _writeBits

	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	STA VDC_DATAHI		; Restore old CR contents

	cli
	PLX
	PLY
	rts
.ends
;****************************************************************
; DBG_PrintHex:							*
;	Print A's value in hex as tiles at DBG_TextDest.	*
;****************************************************************
.section "DBG_PrintHex" force
DBG_PrintHex:
	PHX
	PHY
	TAY
	sei

	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	AND #%11100111
	STA VDC_DATAHI

	ST0 #VDC_REG_MAWR
	LDA DBG_TextDest + LO
	STA VDC_DATALO
	LDA DBG_TextDest + HI
	STA VDC_DATAHI

	ST0 #VDC_REG_VxR
	LDX #1
_writeBits
	LDA NibSwap_Table,Y
	TAY
	AND #$0F

	CMP #$0A
	bcs _notLetter
	clc
	ADC #<(VCEL_SYSFONT + ('0' - ' '))
	bra _setHi
_notLetter
	ADC #<(VCEL_SYSFONT + ('A' - ' ') - $B)
_setHi
	STA VDC_DATALO
	LDA #>(VCEL_SYSFONT)
	ADC #0
	STA VDC_DATAHI

	INC DBG_TextDest + LO
	bne _noWrapSrc
	INC DBG_TextDest + HI
_noWrapSrc
	DEX
	bpl _writeBits

	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	STA VDC_DATAHI		; Restore old CR contents

	cli
	PLY
	PLX
	rts
.ends
;****************************************************************
; DBG_PrintTile:						*
;	Print the null-terminated tile string (two bytes per	*
; entry, each representing a tile number) whose address is in	*
; DBG_TextSrc to the horizontal strip of tiles at DBG_TextDest.	*
;****************************************************************
.section "DBG_PrintTile" force
DBG_PrintTile:
	PHY			; Make sure we don't conflict
	sei			; with VBL/HBL routines
				; communicating with the VDC.
	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	AND #%11100111		; Save the old VDC CR contents,
	STA VDC_DATAHI		; then force +1 increment mode.

	ST0 #VDC_REG_MAWR
	LDA DBG_TextDest + LO
	STA VDC_DATALO
	LDA DBG_TextDest + HI
	STA VDC_DATAHI

	ST0 #VDC_REG_VxR
	CLY
_writeTiles
	LDA (DBG_TextSrc),Y	; Check to see if we've hit the
	INY			; two-byte terminator (zero tile) yet.
	ORA (DBG_TextSrc),Y
	beq _done

	DEY
	LDA (DBG_TextSrc),Y
	STA VDC_DATALO
	INY
	LDA (DBG_TextSrc),Y
	STA VDC_DATAHI
	INY

	INC DBG_TextDest + LO	; Autoincrement the destination
	bne _noWrapSrc		; address for convenience.
	INC DBG_TextDest + HI
_noWrapSrc
	bra _writeTiles		; Copy tiles one by one
_done

	ST0 #VDC_REG_CR
	LDA VDCM_CR + LO
	STA VDC_DATALO
	LDA VDCM_CR + HI
	STA VDC_DATAHI		; Restore old CR contents

	PLY
	cli			; IRQs back on...
	rts
.ends
