;****************************************************************
;			PCE Initialization			*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
.orga $ff00
Secret_BootMsg:	.db "Was auch Immer!"
.orga $ff0f
;****************************************************************
; PCE_Reset:							*
;	Boot up the PCE from an unswapped startup vector.	*
;****************************************************************
.section "PCE_Reset" force
PCE_Reset:
	sei
	cld
	LDA #0
	TAX
	TAY
	DEX
	TXS
.ends
.orga $ff17
; Then fall into the initialization code below
;****************************************************************
; PCE_Initialize:						*
;	Place the hardware in a known state, and then start	*
; execution of the game code. If Y=0 we came from an unswapped	*
; reset vector, otherwise if Y=1 we're operating in a region	*
; requring MCGenjin to swap the data lines.			*
;****************************************************************
.section "PCE_Initialize" force
PCE_Initialize:
	LDA #BANK_HW
	TAM #MPR0
	LDA #BANK_WRAM
	TAM #MPR1
	LDA #BANK_EXRAM
	TAM #MPR2
	LDA #BANK_FRAM
	TAM #MPR3

	LDA #$7F			; MPR 4-5 are swappable OBJ slots.
	TAM #(MPR4 | MPR5)		; Swaps are tracked throungh a stack.
	STZ MCG_LastMap			; $7F lives on the bottom.
	LDA #1
	TAM #MPR6			; MPR 6-7 are fixed to 1-0 for System services
_ClearZP
	STZ $0,X
	DEX
	bne _ClearZP
	STY $0				; Clear ZeroPage, Set Region (Must be Zp $0)

	LDY #0
	LDA #$21
	STA GLO_PointerA + HI
	TXA

	CSH				; Switch into high speed mode before
_ClearRAM				; proceeding to the rest of initialization.
	STA (GLO_PointerA),Y
	INY
	bne _ClearRAM
	LDX GLO_PointerA + HI
	INX
	STX GLO_PointerA + HI
	CPX #$40
	bne _ClearRAM			; Clear RAM

	STA TIMER_EN			; Shut Off Timer
	LDA #(IRQ2 | IRQ1 | IRQTIMER)
	STA IRQ_EN			; Kill all IRQ Hardware
	STA IRQ_STAT			; Ack Timer IRQ

	LDA #$4C			; Initialize all of our "jump to
	STA VBL_VarJump			; whatever address" variables.
	STA GLO_VarJump

	LDY #((12 * 3) - 1)
_InitVDC				; Initialize the VDC using
	LDA VDC_InitData,Y		; the contents of VDC_InitData
	STA VDC_REG
	DEY
	LDA VDC_InitData,Y
	STA VDC_DATALO
	DEY
	LDA VDC_InitData,Y
	STA VDC_DATAHI
	DEY
	bpl _InitVDC

	LDA #(VDC_ENABLE_BG | VDC_ENABLE_SPR | VDC_IRQ_VBL | VDC_IRQ_HBL)
	STA VDCM_CR + LO		; Also preset our VDC CR & MWR mirrors,
	STA VDCB_CR + LO		; the others were safely defaulted
	LDA #VDC_MAP_64x32		; in the zeropage clearing routine.
	STA VDCM_MWR + LO
	STA VDCB_MWR + LO

	jsr VDC_ClearVRAM		; Clear VRAM
	jsr VCE_ClearCRAM		; Clear CRAM

	LDA #(VCE_SMOOTH | VCE_7MHZ)
	STA VCE_CONT			; 7MHz Color Clock, Smoothing
	STZ IRQ_EN			; IRQ Hardware Back On

	jsr MCG_ClearEXRAM		; Clear EXRAM
	LDA #BANK_EXRAM			; Restore MPR2 to the lowest EXRAM bank.
	TAM #MPR2
	jmp Start			; Let's play!

;*******************
; VDC Initial State
;	    $HI,$LO,	DEST
VDC_InitData
	.db $7F,$00,	VDC_REG_SATB
	.db $00,$00,	VDC_REG_DCR
	.db $00,$0C,	VDC_REG_VCR	; Frame Attributes ----
	.db $00,$D0,	VDC_REG_VDW
	.db $1C,$02,	VDC_REG_VPR	; 208 lines tall
	.db $04,$27,	VDC_REG_HDR
	.db $05,$02,	VDC_REG_HSR	; 320 pixels (40 tiles) wide
	.db $00,$10,	VDC_REG_MWR	; 64x32 tile screen
	.db $00,$00,	VDC_REG_BYR
	.db $00,$00,	VDC_REG_BXR	; Initial scroll offset is 0,0
	.db $00,$00,	VDC_REG_RCR	; Raster IRQ out of range on line 0
	.db $00,$CC,	VDC_REG_CR	; BG, SPR, EOF IRQ, and RCR IRQ enabled.
.ends




;****************************************************************
; MCGenjin Header						*
;****************************************************************
.orga $ffd0
MCGenjin_Flag:	.db "MCGENJIN"		; MCGenjin 4MB Plus:
MCGenjin_Rev:	.db 0			; Chip Revision 0
MCGenjin_Pages:	.db $10			; 4MB ROM
MCGenjin_NReg:	.db 0			; US-Region
MCGenjin_UCS0:	.db $20			; 8KB FRAM on User CS0
MCGenjin_UCS1:	.db $14,0,0,0		; 128KB SRAM on User CS1




.orga $ffe0
;****************************************************************
; PCE_ResetSwappedFinish:					*
;	The second half of the swapped reset operations.	*
;****************************************************************
.section "PCE_ResetSwappedFinish" force
PCE_ResetSwappedFinish:
	.db %01000110			; $62 CLA
	.db %01010101			; $AA TAX
	.db %00010101			; $A8 TAY
	.db %01010011			; $CA DEX
	.db %01011001			; $9A TXS
	.db %00010011			; $C8 INY
	.db %01110001,%01111111,$FF	; $8E,$FE,$FF STX $FFFE, Set MCGenjin Region
	jmp PCE_Initialize		; We're back to normal D-Orders!
.ends
.orga $fff0
;****************************************************************
; PCE_ResetSwapped:						*
;	Boot up the PCE from a swapped startup vector.		*
;****************************************************************
.section "PCE_ResetSwapped" force
PCE_ResetSwapped:
	.db %00011110			; $78 sei
	.db %00011011			; $D8 cld
	.db %00110010,%00000111,$FF	; $4C,$E0,$FF jmp PCE_ResetSwappedFinish
.ends
;****************************************************************
; IRQ_Dummy:							*
;	Just a Dummy IRQ.					*
;****************************************************************
.section "IRQ_Dummy" force
IRQ_Dummy:
	rti
.ends
