;****************************************************************
;		     MCGenjin Helper Library			*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
; -MCG_ResolveLo:
;	MCG_Target = MCG_Base + MCG_Offset, using OBJ/LO.
;
; -MCG_TestAllMem:
;	Test all attached FRAM and SRAM on CS0/1, carry is set
;	if everything is working.
; -MCG_ClearEXRAM:
;	Zero out EXRAM / SRAM.

;****************************************************************
; MCG_SwapTable:						*
;	Not a subroutine, but fairly important. A 256-Entry	*
; table containing all values from 0-255, with the order of	*
; the bits reversed (D0 is D7, D1 is D6, etc).			*
;****************************************************************
.section "MCG_SwapTable" force
MCG_SwapTable:
	.db	$00,$80,$40,$C0,$20,$A0,$60,$E0,$10,$90,$50,$D0,$30,$B0,$70,$F0
	.db	$08,$88,$48,$C8,$28,$A8,$68,$E8,$18,$98,$58,$D8,$38,$B8,$78,$F8
	.db	$04,$84,$44,$C4,$24,$A4,$64,$E4,$14,$94,$54,$D4,$34,$B4,$74,$F4
	.db	$0C,$8C,$4C,$CC,$2C,$AC,$6C,$EC,$1C,$9C,$5C,$DC,$3C,$BC,$7C,$FC
	.db	$02,$82,$42,$C2,$22,$A2,$62,$E2,$12,$92,$52,$D2,$32,$B2,$72,$F2
	.db	$0A,$8A,$4A,$CA,$2A,$AA,$6A,$EA,$1A,$9A,$5A,$DA,$3A,$BA,$7A,$FA
	.db	$06,$86,$46,$C6,$26,$A6,$66,$E6,$16,$96,$56,$D6,$36,$B6,$76,$F6
	.db	$0E,$8E,$4E,$CE,$2E,$AE,$6E,$EE,$1E,$9E,$5E,$DE,$3E,$BE,$7E,$FE
	.db	$01,$81,$41,$C1,$21,$A1,$61,$E1,$11,$91,$51,$D1,$31,$B1,$71,$F1
	.db	$09,$89,$49,$C9,$29,$A9,$69,$E9,$19,$99,$59,$D9,$39,$B9,$79,$F9
	.db	$05,$85,$45,$C5,$25,$A5,$65,$E5,$15,$95,$55,$D5,$35,$B5,$75,$F5
	.db	$0D,$8D,$4D,$CD,$2D,$AD,$6D,$ED,$1D,$9D,$5D,$DD,$3D,$BD,$7D,$FD
	.db	$03,$83,$43,$C3,$23,$A3,$63,$E3,$13,$93,$53,$D3,$33,$B3,$73,$F3
	.db	$0B,$8B,$4B,$CB,$2B,$AB,$6B,$EB,$1B,$9B,$5B,$DB,$3B,$BB,$7B,$FB
	.db	$07,$87,$47,$C7,$27,$A7,$67,$E7,$17,$97,$57,$D7,$37,$B7,$77,$F7
	.db	$0F,$8F,$4F,$CF,$2F,$AF,$6F,$EF,$1F,$9F,$5F,$DF,$3F,$BF,$7F,$FF
.ends


;****************************************************************
; MCG_ResolveLo:						*
;	Calculate the OBJ/LO address and mapping parameters	*
; relative to MCG_Base with offset MCG_Offset, storing the	*
; result in MCG_Target.	A is trashed.				*
;****************************************************************
.section "MCG_ResolveLo" force
MCG_ResolveLo:
	clc
	LDA MCG_Base + LO
	ADC MCG_Offset + LO
	STA MCG_Target + LO

	LDA MCG_Base + HI
	ADC MCG_Offset + HI
	CMP #>(ADDR_OBJ_HI - $0100)
	bcc _writeHiAddr
	SBC #>(ADDR_OBJ_HI - ADDR_OBJ_LO)
	sec
_writeHiAddr
	STA MCG_Target + HI

	LDA MCG_Base + BANK
	ADC MCG_Offset + BANK
	CMP #(BANK_ROM + MCBANK_PAGES)
	bcc _writeBank
	SBC #MCBANK_PAGES
	sec
_writeBank
	STA MCG_Target + BANK

	LDA MCG_Base + MCMAP
	ADC MCG_Offset + MCMAP
	STA MCG_Target + MCMAP
	rts
.ends




;****************************************************************
; MCG_TestAllMem:						*
;	Test the attached FRAM and EXRAM. Carry is set on	*
; a proper test pass, and cleared on a failure.			*
;****************************************************************
.section "MCG_TestAllMem" force
MCG_TestAllMem:
	TMA #MPR3
	PHA
	TMA #MPR2
	PHA				; Preserve MPRs
	PHX
	PHY				; Also X,Y (Just in case)
	jsr MCG_TestEXRAM
	LDA #0
	ADC #0
	TAX
	jsr MCG_TestFRAM
	TXA
	ADC #0
	CMP #2
	bne _failure
	sec
	bra _exit
_failure
	clc
_exit
	PLY
	PLX
	PLA
	TAM #MPR2
	PLA
	TAM #MPR3			; Clean up
	rts				; Let's split
.ends
;****************************************************************
; MCG_TestFRAM:							*
;	Ensure proper operation of FRAM, result returned in	*
; the carry flag. Carry set indicates proper operation, clear	*
; indicates a failure during the test. Note that this test	*
; is nondestructive, and the contents of FRAM will be saved.	*
;****************************************************************
.section "MCG_TestFRAM" force
MCG_TestFRAM:
	STZ GLO_PointerA + LO
	LDA #>FRAM_Base
	STA GLO_PointerA + HI

	LDY #BANK_FRAM
	TYA
	TAM #MPR3
_procNextByte
	LDA (GLO_PointerA)		; Push original byte to
	PHA				; stack for preservation
	LDA #1
	STA GLO_TempA			; Initial 1
_iterateBit
	LDA GLO_TempA
	STA (GLO_PointerA)
	CMP (GLO_PointerA)
	bne _failure
	clc
	ROL A
	STA GLO_TempA			; "Wandering 1" test
	bcc _iterateBit

	PLA
	STA (GLO_PointerA)		; Restore original byte

	INC GLO_PointerA + LO
	bne _noHighWrap
	INC GLO_PointerA + HI
_noHighWrap
	LDA GLO_PointerA + HI
	CMP #>(FRAM_Base + $2000)
	bne _noBlockWrap
	INY
	CPY #(BANK_FRAM + FRAM_PAGES)
	beq _pass
	TYA
	TAM #MPR3
	LDA #>FRAM_Base
	STA GLO_PointerA + HI
_noBlockWrap
	bra _procNextByte
_failure
	PLA
	STA (GLO_PointerA)		; Restore original byte
	clc				; even if we hit a failure
	rts
_pass
	sec
	rts
.ends
;****************************************************************
; MCG_TestEXRAM:						*
;	Ensure proper operation of EXRAM, result returned in	*
; the carry flag. Carry set indicates proper operation, clear	*
; indicates a failure during the test.				*
;****************************************************************
.section "MCG_TestEXRAM" force
MCG_TestEXRAM:
	jsr MCG_ClearEXRAM		; Clear EXRAM beforehand
	STZ GLO_PointerA + LO
	LDA #>EXRAM_Base
	STA GLO_PointerA + HI

	LDY #BANK_EXRAM
	TYA
	TAM #MPR2
_procNextByte
	LDA (GLO_PointerA)		; This should be zero unless
	bne _failure			; some pins got tied together.
	LDA #1
	STA GLO_TempA
_iterateBit
	LDA GLO_TempA
	STA (GLO_PointerA)
	CMP (GLO_PointerA)
	bne _failure
	clc
	ROL A
	STA GLO_TempA			; "Wandering 1" test
	bcc _iterateBit

	LDA #$FF
	STA (GLO_PointerA)		; Force FF before moving on

	INC GLO_PointerA + LO
	bne _noHighWrap
	INC GLO_PointerA + HI
_noHighWrap
	LDA GLO_PointerA + HI
	CMP #>(EXRAM_Base + $2000)
	bne _noBlockWrap
	INY
	CPY #(BANK_EXRAM + EXRAM_PAGES)
	beq _pass
	TYA
	TAM #MPR2
	LDA #>EXRAM_Base
	STA GLO_PointerA + HI
_noBlockWrap
	bra _procNextByte
_failure
	clc
	rts
_pass
	sec
	rts
.ends




;****************************************************************
; MCG_ClearEXRAM:						*
;	Zero all of EXRAM.					*
;****************************************************************
.section "MCG_ClearEXRAM" force
MCG_ClearEXRAM:
	STZ GLO_PointerA + LO
	LDA #>EXRAM_Base
	STA GLO_PointerA + HI

	LDY #BANK_EXRAM
	TYA
	TAM #MPR2
_procNextByte
	LDA #0
	STA (GLO_PointerA)

	INC GLO_PointerA + LO
	bne _noHighWrap
	INC GLO_PointerA + HI
_noHighWrap
	LDA GLO_PointerA + HI
	CMP #>(EXRAM_Base + $2000)
	bne _noBlockWrap
	INY
	CPY #(BANK_EXRAM + EXRAM_PAGES)
	beq _done
	TYA
	TAM #MPR2
	LDA #>EXRAM_Base
	STA GLO_PointerA + HI
_noBlockWrap
	bra _procNextByte
_done
	rts
.ends
