;****************************************************************
;			  PCE VDC Helpers			*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
; -VDC_MakeVREQ_B:
; -VDC_MakeVREQ:
;	Requests go in...
;
; -VDC_FlushVREQs:
;	...data goes out

;****************************************************************
; VDC_MakeVREQ_B:						*
;	Performs the same action as VDC_MakeVREQ, except using	*
; a fixed transfer block which resides at VDAT_BlockSrc.	*
;****************************************************************
.section "VDC_MakeVREQ_B" force
VDC_MakeVREQ_B:
	PHY

	LDY #(VREQ_LEN - 1)
_copyBlock			; | Must be done since WLA assumes
	LDA (VDAT_BlockSrc),Y	; v 0-FF,Y is $00xx,Y instead of ZP
	STA VDAT_Type + ADDR_RAM,Y
	DEY
	bpl _copyBlock		; | Then fall into VDC_MakeVREQ.
	PLY			; v
.ends
;****************************************************************
; VDC_MakeVREQ:							*
;	Adds the VRAM transfer request defined in VDAT_x to the	*
; current VREQ ringbuffer, will wait if the buffer is full.	*
;****************************************************************
.section "VDC_MakeVREQ" force
VDC_MakeVREQ:
	PHY
	PHX

	LDY VDC_TxRing_I
	TYA
	INA
	AND #VREQ_RINGMASK
_waitRingReady
	CMP VDC_TxRing_O
	beq _waitRingReady

	LDA VDAT_Type
	STA VREQ_TxRing_Type,Y

	LDA VDAT_Src + LO
	STA VREQ_TxRing_Source_Lo,Y
	LDA VDAT_Src + HI
	STA VREQ_TxRing_Source_Hi,Y
	LDA VDAT_Src + BANK
	STA VREQ_TxRing_Source_Bank,Y

	LDA VDAT_Src + MCMAP	; We MUST pre-swap the McGenjin
	LDX SysRegion		; mapping write before enqueuing
	beq _noSwapMap		; the transfer request.
	TAX
	LDA.w MCG_SwapTable,X
_noSwapMap
	STA VREQ_TxRing_Source_Map,Y

	LDA VDAT_Dest + LO
	STA VREQ_TxRing_Dest_Lo,Y
	LDA VDAT_Dest + HI
	STA VREQ_TxRing_Dest_Hi,Y

	LDA VDAT_Size + HI
	STA VREQ_TxRing_Size_Hi,Y
	LDA VDAT_Size + LO
	STA VREQ_TxRing_Size_Lo,Y
	bne _noAdjustLen
	LDA VREQ_TxRing_Size_Hi,Y
	DEA
	STA VREQ_TxRing_Size_Hi,Y
_noAdjustLen

	LDA VDC_TxRing_I
	INA
	AND #VREQ_RINGMASK
	STA VDC_TxRing_I
	PLX
	PLY
	rts
.ends




;****************************************************************
; VDC_FlushVREQs:						*
;	Handle all outstanding VRAM transfer requests. Should	*
; be called only during VBL as to not interfere with any raster *
; effects which may adjust the VDC registers. The current state	*
; of MPR4, MPR5, and the MCGenjin bank must be saved.		*
;****************************************************************
.section "VDC_FlushVREQs" force
VDC_FlushVREQs:
	LDX VDC_TxRing_O
	CPX VDC_TxRing_I
	bne _initTransfers
	rts

_initTransfers
	LDA VDCB_CR + HI	; Before we start, we need to be sure that
	AND #%11100111		; the data read/write step bits in the VDC
	STA VDCB_CR + HI	; CR backup are cleared.

_doTransfers
	LDA VREQ_TxRing_Type,X	; First fetch the type, keep a backup
	TAY			; in Y for when we do final decoding later.
	AND #VDC_TRANSFER_STEPMASK
	STA VREQ_TxRing_Type,X	; Type now only contains the write step.

	ST0 #VDC_REG_CR
	LDA VDCB_CR + LO
	STA VDC_DATALO
	LDA VDCB_CR + HI
	ORA VREQ_TxRing_Type,X
	STA VDC_DATAHI		; Set Transfer Step Mode

	ST0 #VDC_REG_MAWR
	LDA VREQ_TxRing_Dest_Lo,X
	STA VDC_DATALO
	LDA VREQ_TxRing_Dest_Hi,X
	STA VDC_DATAHI		; Set Destination VRAM Address and
	ST0 #VDC_REG_VxR	; select VxR for free writing.

	TYA			; We can now start the transfer, first
	bpl _doCopy		; check if it's a copy or a fill.

_doFill				; -- HANDLE FILL --
	jsr VDC_VREQ_Fill
	TXA
	INA
	AND #VREQ_RINGMASK
	TAX
	STX VDC_TxRing_O
	CPX VDC_TxRing_I
	bne _doTransfers
	rts

_doCopy				; -- HANDLE COPY --
	AND #VDC_TRANSFER_MODEMASK
	TAY
	LDA VDC_VREQ_ModeLo,Y
	STA VBL_JumpDest + LO	
	LDA VDC_VREQ_ModeHi,Y
	STA VBL_JumpDest + HI

	LDA VREQ_TxRing_Source_Bank,X
	TAM #MPR4		; OBJ and MCGenjin states have (or should
	INA			; have) been saved before calling this routine.
	TAM #MPR5
	LDA VREQ_TxRing_Source_Map,X
	STA MCG_LastMap
	STA MCGENJIN_BANK	; <- The MCGenjin bank select write
				; MUST be pre-swapped during enqueuing.

	jsr VBL_VarJump + ADDR_RAM

	TXA
	INA
	AND #VREQ_RINGMASK
	TAX
	STX VDC_TxRing_O
	CPX VDC_TxRing_I
	bne _doTransfers
	rts
.ends
;****************************************************************
; VRAM Copy Routines						*
;****************************************************************
VDC_VREQ_ModeLo .db	<VDC_VREQ_CopyQBG,	<VDC_VREQ_CopyDBG
		.db	<VDC_VREQ_Copy,		<VDC_VREQ_CopyFast
VDC_VREQ_ModeHi .db	>VDC_VREQ_CopyQBG,	>VDC_VREQ_CopyDBG
		.db	>VDC_VREQ_Copy,		>VDC_VREQ_CopyFast
;----------------
; VDC_VREQ_Copy:
;	Copy the data at VREQ_SOURCE to VREQ_DEST with length
; VREQ_SIZE in BYTES (must be even). Current VREQ should be in X.
;----------------------------------------------------------------
.section "VDC_VREQ_Copy" force
VDC_VREQ_Copy:
	LDA VREQ_TxRing_Source_Lo,X
	STA VDC_CopySrc + LO
	LDA VREQ_TxRing_Source_Hi,X
	STA VDC_CopySrc + HI

	LDY VREQ_TxRing_Size_Lo,X
_copyBytes
	LDA (VDC_CopySrc),Y
	STA VDC_DATALO
	DEY
	LDA (VDC_CopySrc),Y
	STA VDC_DATAHI
	DEY
	bne _copyBytes

	DEC VREQ_TxRing_Size_Hi,X
	bmi _done
	INC VDC_CopySrc + HI
	bra _copyBytes
_done
	rts
.ends
;----------------
; VDC_VREQ_CopyFast:
;	Execute the list of ST1/2 opcodes at VREQ_SOURCE, their
; immediate data will be stored at VREQ_DEST. This list must be
; terminated by an RTS.
;----------------------------------------------------------------
.section "VDC_VREQ_CopyFast" force
VDC_VREQ_CopyFast:
	LDA VREQ_TxRing_Source_Lo,X
	STA VDC_CopySrc + LO
	LDA VREQ_TxRing_Source_Hi,X
	STA VDC_CopySrc + HI

	jmp (VDC_CopySrc)
.ends
;----------------
; VDC_VREQ_CopyDBG:
;	Copy the data at VREQ_SOURCE to VREQ_DEST with length
; VREQ_SIZE in BYTES. During the copy operation, the data will
; be automatically expanded from 1bpp to a 2bpp background tile.
; Current VREQ should be in X.
;----------------------------------------------------------------
.section "VDC_VREQ_CopyDBG" force
VDC_VREQ_CopyDBG:
	LDA VREQ_TxRing_Source_Lo,X
	STA VDC_CopySrc + LO
	LDA VREQ_TxRing_Source_Hi,X
	STA VDC_CopySrc + HI

_copyBytes			; We'll be copying the 1bpp data
	LDA (VDC_CopySrc)	; directly into the first bitplane of
	STA VDC_DATALO		; each 2bpp tile.
	ST2 #0

	DEC VREQ_TxRing_Size_Lo,X
	bne _calcNextOffset
	DEC VREQ_TxRing_Size_Hi,X
	bmi _done		; Have we run out of source data?
_calcNextOffset
	INC VDC_CopySrc + LO
	bne _copyBytes
	INC VDC_CopySrc + HI
	bra _copyBytes		; If not, adjust our copy offset and continue.
_done
	rts
.ends
;----------------
; VDC_VREQ_CopyQBG:
;	Copy the data at VREQ_SOURCE to VREQ_DEST with length
; VREQ_SIZE in BYTES. During the copy operation, the data will
; be automatically expanded from 1bpp to a 4bpp background tile.
; Current VREQ should be in X.
;----------------------------------------------------------------
.section "VDC_VREQ_CopyQBG" force
VDC_VREQ_CopyQBG:
	LDA VREQ_TxRing_Source_Lo,X
	STA VDC_CopySrc + LO
	LDA VREQ_TxRing_Source_Hi,X
	STA VDC_CopySrc + HI

	LDY #8
_copyBytes			; We'll be copying the 1bpp data
	LDA (VDC_CopySrc)	; directly into the first bitplane of
	STA VDC_DATALO		; each 4bpp tile.
	ST2 #0

	DEY
	bne _notPadTime		; As the 4bpp tiles are stored as two 
	.repeat 8		; 2bpp tiles, we must zero out the 'second'
		ST1 #0		; tile before proceeding to copy the
		ST2 #0		; next chunk of data.
	.endr
	LDY #8
_notPadTime
	DEC VREQ_TxRing_Size_Lo,X
	bne _calcNextOffset
	DEC VREQ_TxRing_Size_Hi,X
	bmi _done		; Have we run out of source data?
_calcNextOffset
	INC VDC_CopySrc + LO
	bne _copyBytes
	INC VDC_CopySrc + HI
	bra _copyBytes		; If not, adjust our copy offset and continue.
_done
	rts
.ends
;----------------
; VDC_VREQ_Fill:
;	Fill VRAM with a 16-Bit value stored in VREQ_SOURCE with
; length VREQ_SIZE in WORDS, current VREQ should be in X.
;----------------------------------------------------------------
.section "VDC_VREQ_Fill" force
VDC_VREQ_Fill:
	LDA VREQ_TxRing_Source_Lo,X
	LDY VREQ_TxRing_Source_Hi,X

_writeWords
	STA VDC_DATALO
	STY VDC_DATAHI
	
	DEC VREQ_TxRing_Size_Lo,X
	bne _writeWords
	DEC VREQ_TxRing_Size_Hi,X
	bmi _done
	bra _writeWords
_done
	rts
.ends




;****************************************************************
; VDC_ClearVRAM:						*
;	Clear all of VRAM. This should only be called once	*
; from the system initialization routine.			*
;****************************************************************
.section "VDC_ClearVRAM" force
VDC_ClearVRAM:
	ST0 #VDC_REG_MAWR	; Set VDC write address to $0
	ST1 #0
	ST2 #0
	ST0 #VDC_REG_VxR

	LDX #<VDC_VRAM_SIZE	; 32K Words to Clear
	LDY #>VDC_VRAM_SIZE	; Write the low byte first, it will remain
	ST1 #0			; unchanged throughout the further writes...
_clearWords
	ST2 #0			; ...so we can just write the high byte.
	DEX
	bne _clearWords
	DEY
	bne _clearWords
	rts
.ends
